@@ -8,9 +8,7 @@ This crate provide a unit struct with many methods to operate uart 8250.
 
 #![no_std]
 
-extern crate volatile_register;
-
 pub mod registers;
 pub mod uart;
 
-pub use uart::{ChipFIFOInfo, InterruptType, MmioUart8250, Parity};
+pub use uart::{ChipFifoInfo, InterruptType, MmioUart8250, Parity};
